//===- subzero/runtime/szrt.c - Subzero runtime source ----------*- C++ -*-===//
//
//                        The Subzero Code Generator
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// This file implements wrappers for particular bitcode instructions
// that are too uncommon and complex for a particular target to bother
// implementing directly in Subzero target lowering.  This needs to be
// compiled by some non-Subzero compiler.
//
//===----------------------------------------------------------------------===//

#include <stdint.h>

// TODO(stichnot): The various NaN cross tests try to map Subzero's
// undefined behavior to the same as llc's undefined behavior, as
// observed by the cross tests.  This will have to be kept up to date
// with any future changes to llc, and may also have to be different
// for different targets.  It would be better to find a more
// appropriate set of llc options when building the Subzero runtime.
//
// We test for NaN using "Value==Value" instead of using isnan(Value)
// to avoid an external dependency on fpclassify().

uint32_t __Sz_fptoui_f32_i32(float Value) {
  if (Value == Value) // NaNaN
    return (uint32_t)Value;
  return 0x80000000;
}

uint32_t __Sz_fptoui_f64_i32(double Value) {
  if (Value == Value) // NaNaN
    return (uint32_t)Value;
  return 0x80000000;
}

uint64_t __Sz_fptoui_f32_i64(float Value) { return (uint64_t)Value; }

uint64_t __Sz_fptoui_f64_i64(double Value) { return (uint64_t)Value; }

int64_t __Sz_fptosi_f32_i64(float Value) { return (int64_t)Value; }

int64_t __Sz_fptosi_f64_i64(double Value) { return (int64_t)Value; }

float __Sz_uitofp_i32_f32(uint32_t Value) { return (float)Value; }

float __Sz_uitofp_i64_f32(uint64_t Value) { return (float)Value; }

double __Sz_uitofp_i32_f64(uint32_t Value) { return (double)Value; }

double __Sz_uitofp_i64_f64(uint64_t Value) { return (double)Value; }

float __Sz_sitofp_i64_f32(int64_t Value) { return (float)Value; }

double __Sz_sitofp_i64_f64(int64_t Value) { return (double)Value; }

// Other helper calls emitted by Subzero but not implemented here:
// Compiler-rt:
//   __udivdi3     - udiv i64
//   __divdi3      - sdiv i64
//   __umoddi3     - urem i64
//   __moddi3      - srem i64
//   __popcountsi2 - call @llvm.ctpop.i32
//   __popcountdi2 - call @llvm.ctpop.i64
// libm:
//   fmodf - frem f32
//   fmod  - frem f64
// libc:
//   setjmp  - call @llvm.nacl.setjmp
//   longjmp - call @llvm.nacl.longjmp
//   memcpy  - call @llvm.memcpy.p0i8.p0i8.i32
//   memmove - call @llvm.memmove.p0i8.p0i8.i32
//   memset  - call @llvm.memset.p0i8.i32
// unsandboxed_irt:
//   __nacl_read_tp
